/*
    Synth Lua - Formant speech synthesizer
    Copyright (C) 2013 Belousov Oleg <belousov.oleg@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "noise.h"
#include "cordic.h"
#include "formant.h"
#include "glotal.h"
#include "nasal.h"
#include "frication.h"

#define MS(x)	(x * RATE / 2000)

int fd;

char render(tract_t *tract) {
    int x, y = 0;

    x = noise() >> 16;
    x *= tract->aa;
    x >>= 8;

    glotal_render(&x);
    formant_render(&x, &y);
    nasal_render(tract, &x, &y);
    frication_render(tract, &x);

    return x / CORDIC_MUL * 127;
}

void transition(tract_t *from, tract_t *to, int time) {
    char	out;
    int 	i, n;
    tract_t	tract = *from;

    for (i = 0; i < time; i++) {
	tract.ga = from->ga + (to->ga - from->ga) * i / time;
	tract.aa = from->aa + (to->aa - from->aa) * i / time;

	tract.na = from->na + (to->na - from->na) * i / time;
	tract.nf = from->nf + (to->nf - from->nf) * i / time;

	tract.fa = from->fa + (to->fa - from->fa) * i / time;

	for (n = 0; n < 4; n++)
	    tract.f[n] = from->f[n] + (to->f[n] - from->f[n]) * i / time;

	formant_set(&tract);

	out = render(&tract);
	write(fd, &out, sizeof(out));
    }
}

void sustain(tract_t *tract, int time) {
    char	out;
    int 	i;

    for (i = 0; i < time; i++) {
	out = render(tract);
	write(fd, &out, sizeof(out));
    }
}

void fade_in(tract_t *tract, int time) {
    tract_t	t_temp = *tract;

    t_temp.ga = 0;
    t_temp.aa = 0;
    t_temp.fa = 0;

    transition(&t_temp, tract, time);
}

void fade_out(tract_t *tract, int time) {
    tract_t	t_temp = *tract;

    t_temp.ga = 0;
    t_temp.aa = 0;
    t_temp.fa = 0;

    transition(tract, &t_temp, time);
}

int main() {
    int		i;

    fd = open("test.raw", O_WRONLY | O_CREAT, 0644);

    tract_t	t_inhale = { .ga = 0, .aa = 500, .f = { 880, 1075, 2350, 3050} };
    tract_t	t_exhale = { .ga = 0, .aa = 250, .f = { 500, 800, 1100, 2325} };
    tract_t	t_silent = { .ga = 0, .aa = 5, .f = { 500, 800, 1100, 2325} };

    tract_t	t_ee = { .ga = 50, .aa = 250, .f = { 310, 2000, 3010, 3700} };
    tract_t	t_a = { .ga = 50, .aa = 250, .f = { 730, 1700, 2500, 3400} };
    tract_t	t_oa = { .ga = 25, .aa = 250, .f = { 530, 950, 2400, 3200} };
    tract_t	t_ll = { .ga = 25, .aa = 250, .f = { 560, 850, 2600, 3600} };

    tract_t	t_o = { .ga = 8,	.aa = 250,	.f = { 750, 1050, 2400, 3200} };
    tract_t	t_m = { .ga = 10*2,	.aa = 250,	.f = { 170, 1100, 2600, 3200}, .nf = 2900, .na = 200 };
    tract_t	t_n = { .ga = 10*4,	.aa = 250,	.f = { 200, 1550, 2500, 4500}, .nf = 1550, .na = 200 };
    tract_t	t_s = { 		.aa = 5,	.f = { 2057, 3170, 4079, 4000}, .ff = 3170, .fa = 30 };
    tract_t	t_t = { 		.aa = 5,	.f = { 2057, 3170, 4079, 4000}, .ff = 3125, .fa = 90 };
    tract_t	t_r = { .ga = 5,	.aa = 100, 	.f = { 580, 1200, 1500, 4700} };

    tract_vp = 2;
    tract_vr = 7;

    /*
    fade_in(&t_inhale, MS(500));
    fade_out(&t_inhale, MS(500));
    fade_in(&t_exhale, MS(750));
    fade_out(&t_exhale, MS(750));
    */

    for (i = 0; i < 15; i++) {
	tract_gf = 70 + (noise() % 10) * 5;

	fade_in(&t_m, MS(150));
	transition(&t_m, &t_o, MS(50));
	sustain(&t_o, MS(150));

	transition(&t_o, &t_n, MS(150));
	sustain(&t_n, MS(150));
	fade_out(&t_n, MS(100));

	fade_in(&t_s, MS(150));
	fade_out(&t_s, MS(75));

	fade_in(&t_t, MS(50));
	fade_out(&t_t, MS(20));

	fade_in(&t_r, MS(50));
	sustain(&t_r, MS(150));
	fade_out(&t_r, MS(100));

	sustain(&t_silent, MS(500));
    }

    return 0;
}
